<?php
// $Id: shepherd.admin.inc,v 1.1 2010/12/16 19:46:48 emackn Exp $
/**
 * @file
 * Admin settings, construct forms for server, client, and component admin sections.
 */

/**
 * Admin settings for shepherd components
 *
 * @return 
 *   component form array
 */
function admin_settings_shepherd() {
  
  $settings_form = array();
  $settings_form['shepherd_component_toggle'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Enable components'),
    '#default_value' => variable_get('shepherd_component_toggle', array('client')),
    '#options' => array(
      'client' => t('Client'),
      'server' => t('Server'),      
    ),
    '#description' => t('<ul><li>Client - Machine is part of the flock, reports status to a Shepherd server.<li/>
      <li>Server - Receieve reports from clients.<li/></ul>A single machine can be both, one or none.  Saving this page will force a menu rebuild.'),
  );
  
  $form = system_settings_form($settings_form);
  
  $form['#submit'][] = 'shepherd_import_content_type';

  // rebuild menu so server tab is removed
  $form['#submit'][] = 'menu_rebuild';

  return $form;
}

// the component default tab
function admin_settings_shepherd_components() {
  return admin_settings_shepherd();
}

/**
 * Admin settings for shepherd client settings.
 *
 * @return 
 *   client form array
 */
function admin_settings_shepherd_client() {
  global $base_url;
  
  $settings_form = array();
  
  $settings_form['shepherd_client_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Client Information'),
    '#description' => t('Vital client information that will be passed to the server when connecting.'),
    '#weight' => -1,
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $settings_form['shepherd_client_info']['shepherd_client_sitename'] = array(
    '#type' => 'markup',
    '#value' => "<div class='form-item shepherd-readonly-info'><label>Site Name: </label>". variable_get('site_name', "Not Set") ."</div>",
  );
  $settings_form['shepherd_client_info']['shepherd_client_vhost'] = array(
    '#type' => 'markup',
    '#value' => "<div class='form-item shepherd-readonly-info'><label>Virtual Host: </label>". $base_url ."</div>",
  );  
  $settings_form['shepherd_client_info']['shepherd_client_ip'] = array(
    '#type' => 'markup',
    '#value' => "<div class='form-item shepherd-readonly-info'><label>IP Address: </label>". _shepherd_ip_for_host() ."</div>",
  );

  $server_url = (is_shepherd() ? $base_url ."/" : "http://remote-shepherd-server.com/");

  $settings_form['shepherd_server_address'] = array(
    '#title' => t('Server Address / Hostname'),
    '#type' => 'textfield',
    '#description' => t('HTTP web address of Shepherd server.  (http://www.example.com/) Include trailing slash.'),
    '#default_value' => variable_get('shepherd_server_address', $server_url),
  );

  $settings_form['shepherd_server_post_path'] = array(
    '#title' => t('Path of POST URL.'),
    '#type' => 'textfield',
    '#description' => t('Appended to the server address creating a complete post URL.'),
    '#default_value' => variable_get('shepherd_server_post_path', 'shepherd/services/client-update'),
  );  

  $settings_form['shepherd_check_server_connection'] = array(
    '#type' => 'submit',
    '#value' => t('Test Shepherd Connection'),
    '#submit' => array('shepherd_client_check_server_connection'),
  );

  $form = system_settings_form($settings_form);
  
  // test client connection on save.
  $form['#submit'][] = 'shepherd_client_check_server_connection';
  
  return $form;
}

/**
 * callback function that tests the clients connection to the server
 *
 * @return 
 *   none, sets a drupal message on the form for success or failure.
 */
function shepherd_client_check_server_connection($form, &$form_state) {
  
  $base = variable_get('shepherd_server_address', FALSE);
  $path = 'shepherd/services/client-ping';
 
  if ($base) {
    $ch = curl_init($base . $path);
    curl_setopt($ch, 'CURLOPT_FRESH_CONNECT', TRUE);
    curl_exec($ch);
    $response_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);
  }
  
  if ($response_code == 200) {
    drupal_set_message('Connection successful.  The next cron run will report any necessary module updates to the Shepherd.');
  }
  else {
    drupal_set_message('Connection denied to Shepherd server, is your client authorized to connect?', 'error');
  }
}

/**
 * Admin settings for shepherd server settings.
 *
 * @return 
 *   server settings form array
 */
function admin_settings_shepherd_server() {
  $settings_form = array();
  
  $settings_form['shepherd_ip_whitelist'] = array(
    '#title' => 'IPAdress Whitelist',
    '#type' => 'textarea',
    '#description' => t('Enter IP Addresses that are ALLOWED to connect to the server.  One address per line.'),
    '#default_value' => variable_get('shepherd_ip_whitelist', '127.0.0.1'),    
  );
 
  $settings_form['shepherd_sheep_update_threshold'] = array(
    '#type' => 'select',
    '#title' => t('Update threshold'),
    '#default_value' => variable_get('shepherd_sheep_update_threshold', '1 day'),
    '#options' => array(
      '1 day' => t('1 day'),
      '2 days' => t('2 days'),
      '3 days' => t('3 days'),
      '5 days' => t('5 days'),
      '1 week' => t('1 week'),
      '2 weeks' => t('2 weeks'),
      '1 month' => t('1 month'),
    ),
    
    '#description' => t('Set the allowable age of the last client update.  Mark clients that have not meet the age requirement on the Shepherd flock report.'),
  );
    
  return system_settings_form($settings_form);
}

/**
 * submit callback being added to the component save function
 * import the content type when the server component is enabled for the first time.
 * @param
 *   
 * @link http://groups.drupal.org/node/5272#comment-87285 CCK import thread @endlink
 * @return
 *   None
 */
function shepherd_import_content_type($form, &$form_state) {
  // skip if type exists
  // SELECT type FROM {node_type} WHERE type = 'sheep'
  $sql = "SELECT 1 FROM {node_type} WHERE type = 'sheep'";
  $result = db_query($sql);

  if (!db_result($result) && is_shepherd()) {
    if (module_exists('content_copy')) {
      // do import
      $content_type = file_get_contents( drupal_get_path('module', 'shepherd') ."/cck/sheep.cck");
      $form_state = array(
        'values' => array(
             'type_name' => '<create>',
             'macro' => $content_type,
       ),
      );
  
      drupal_execute("content_copy_import_form", $form_state);
      content_clear_type_cache();
      drupal_set_message('Sheep content type successfully added');
    }
    else {
      drupal_set_message('Shepherd temporarily needs the Content copy module enabled to create the sheep content type.  
      Please enable the module and re-save the components.', 'error');      
    }
  }
  // type already exists, no type needed.
}