<?php
// $Id: shepherd.server.inc,v 1.1.2.1 2010/12/17 20:09:57 emackn Exp $
/**
 * @file
 * Shepherd Server callbacks for client update, client test ping, and reporting.
 */

/**
 * callback that waits for a client http POST.
 * From POST, we build a node out of the info and 
 * make it available to the shepherd server.
 *
 *
 * @return
 *   None presently, maybe it should return result of drupal_execute?
 */
function shepherd_server_receive_client_update() {
  global $user;
  $user = user_load(array(uid => 1));

  $client   = $_POST['client_name'];
  $vhost    = $_POST['client_baseurl'];
  $ip       = $_POST['client_ip'];
  $hashkey  = _shepherd_sever_generate_hashkey($client, $vhost, $ip);
  $data     = $_POST['data'];
  
  $node = array('type' => 'sheep');
  
  // does the client already exist?
  $existing_node_id = shepherd_client_check($hashkey);
  if ($existing_node_id) {
    $node = node_load($existing_node_id);
  }
    
  // move to other function
  module_load_include('inc', 'node', 'node.pages');
  $form_state = array();
  $form_state['node'] = (array) $edit;
  $form_state['values']['title']                              = $client;
  $form_state['values']['field_ipaddress'][0]['value']        = $ip;
  $form_state['values']['field_vhost'][0]['value']            = $vhost;  
  $form_state['values']['field_hashkey'][0]['value']          = $hashkey;
  $form_state['values']['field_security_updates'][0]['value'] = $data;
  $form_state['values']['op'] = t('Save');

  drupal_execute('sheep_node_form', $form_state, (object)$node);
  $errors = form_get_errors();
  
  if (count($errors)) {
    watchdog('shepherd', "Save attempt for \"@client - @vhost - @ip\" has failed. Errors: @errors", 
      array(
        '@client' => $client,
        '@vhost'  => $vhost,
        '@ip'     => $ip,
        '@errors' => $errors,
      )
    );
  } 
}

/**
 * Check white list for allowed connections and
 * respond with header and text
 *
 * @return
 *   None, returns 200 or 401 http headers to be read by client.
 */
function shepherd_server_client_ping_response() {
  $connecting_ip = $_SERVER['REMOTE_ADDR'];
  $whitelist = explode("\r\n", variable_get('shepherd_ip_whitelist', ""));
  $allowed = in_array($connecting_ip, $whitelist);
 
  if ($allowed) {
     header('HTTP/1.1 200 OK');
     echo 'allowed';
  }
  else {
     header('HTTP/1.1 401 Unauthorized');
     echo 'not allowed';
  }
  
  exit;
}

/**
 * Look for existing sheep nodes by the hashkey
 * @param $hashkey
 *   key created from the ip address, vhost, and site name of a client
 *
 * @return
 *   TRUE if hashkey is found.
 */
function shepherd_client_check($hashkey) {
  $query = "SELECT nid FROM {content_type_sheep} WHERE field_hashkey_value = '%s'";
  
  $result = db_query($query, $hashkey);
  if ($result) {
    return db_result($result);
  }
  
  return FALSE;
}

/**
 * display callback for the Shepherd flock report
 *
 * @return
 *   themed report page
 */
function shepherd_server_flock_report() {
  $report = "<div id='flock-report'>";

  $sql = "SELECT nid FROM {node} as n WHERE type = 'sheep' ORDER BY n.title ASC";
  $result = db_query($sql);
  
  while ($row = db_fetch_array($result)) {
    $node = node_load($row['nid']);
    $report .= theme('flock_report', $node);
  }

  $report .= "</div>";
  
  return $report;
}

/**
 * generate a key that can be used to quickly look up this node.
 * possibly add index via install hook to this table after content type import.
 *
 *
 */
function _shepherd_sever_generate_hashkey($client_name, $url, $ip) {
  return md5($client_name ."###". $url ."###". $ip);
}

